/****************************************************************************
DVI-D transmitter test for ARTY S7-S50

2021.4 Tatsuyuki Sato

note:

TMDSo͊蓖
+----+------------+-----------+----------------------------+
|PMOD|VACU| IOhCo|f\[X                  |
+----+------------+-----------+----------------------------+
| JA | OSERDESE2  | TMDS_33   |eXgp^[              |
| JB | FAB+ODDR   | TMDS_33   |JATMDSfBXpeBx|
| JC | FAB+ODDR   | LVCMOS_33 |JATMDSM]x    |
+----+------------+-----------+----------------------------+
VACUAIOhCo̓gbvW[̃p[^[őIł܂B

TMDS_33/LVCMOS_33 ύXꍇɂ.xdc t@C̕ύXKvłB

JB,JC̉f\[X debug_select1 , debug_select2 őIł܂B

+------+---------------------------------------+
|SW[0] | eXgp^[I                    |
+------+---------------------------------------+
| OFF  | HDJ[o[                          |
| ON   | J[Of[V                  |
+------+---------------------------------------+

+------+---------------------------------------+
|LED[3]| 1Hz NbNgmFpM          |
+------+---------------------------------------+

includet@C؂ւ邱Ƃŏo̓[hÓIɕύX邱Ƃo܂B

****************************************************************************/

`timescale 1 ps / 1 ps

/****************************************
 TOP module
****************************************/
module cq_dvitx3 #
(
  // 1nd PMOD JA-HDMI
  JA_TMDS_MODE = "TMDS_33",   // GOOD,SXGA OK
  
  // 2nd PMOD JB-HDMI
  JB_TMDS_MODE = "TMDS_33",   // GOOD,SXGA OK
// JB_TMDS_MODE = "PSEDO_PP", // GOOD,SXGA OK
// JB_TMDS_MODE = "PSEDO_OD", // BAD

  // 3nd PMOD JC-HDMI
  JC_TMDS_MODE = "PSEDO_PP" // GOOD,SXGA OK
// JC_TMDS_MODE = "PSEDO_OD" // BAD
) (
// clock
  input    wire   CLK100MHZ, // 100MHz
// PMOD
  output wire [7:0] ja,
  output wire [7:0] jb,
  output wire [7:0] jc,
  input  wire [7:0] jd,
// switch
  input  wire [3:0] sw,
// LED
  output  wire led0_r,led0_g,led0_b,
  output  wire led1_r,led1_g,led1_b,
  output  wire [3:0] led,
// buttons
  input  wire [3:0] btn
);

//******************************************************************//
// output video mode and clock generator defines
//******************************************************************//
`include "vga_pll_pll.vh"
//`include "vic03_pll_pll.vh"
//`include "xga_pll_pll.vh"
//`include "sxga_pll_pll.vh"
//`include "uxga_pll_pll.vh"
//`include "fhd_pll_pll.vh"

//******************************************************************//
// 27MHz base clock Generation
//******************************************************************//
wire pll27m;
wire pll270m;
wire pll27m_locked;
pll100m27m pll100m27m_inst(
  .CLK100M_I(CLK100MHZ),
  .CLK27M_O(pll27m),
  .CLK270M_O(pll270m),
  .reset(1'b0),
  .locked(pll27m_locked)
);

wire clk27m        = pll27m;
wire clk27m_locked = pll27m_locked;

wire  pll270m_g;
BUFG bufg_pll270m(.I(pll270m),.O(pll270m_g));

//******************************************************************//
// video pixel clock & I/O clock Generation
//******************************************************************//
wire pclk_locked;
wire pclk_ready;
wire pclk;

// for OSERDES-DDR mode
wire pclkx5,pclkx1;

// for FAB-OBUFDS mode
wire pclkx5p,pclkx5n;

//******************************************************************//
// pixes clock generation with 7series MMCM
//******************************************************************//
wire mmcm_pclkx5;
wire mmcm_pclkx5p,mmcm_pclkx5n;
wire mmcm_pclkx2,mmcm_pclkx1;
wire mmcm_clkfb;

// MMCM BASED CLOCK GENERATOR
`PCLK_PLL_MODULE PCLK_GEN(    // clk27m25p2m PCLK_GEN(
  .CLK_IN(`PLL_CLOCK_SOURCE), // clk32m / clk27m / clkdcm / clkdcmsp
  .clkfb_in(mmcm_clkfb),
  .clkfb_out(mmcm_clkfb),
//
  .CLK_X5(mmcm_pclkx5),
  .CLK_X5P(mmcm_pclkx5p),
  .CLK_X5N(mmcm_pclkx5n),
  .CLK_X2(),
  .CLK_X1(mmcm_pclkx1),
  .reset(~clk27m_locked || ~pll27m_locked),
  .locked(pclk_ready)
);
assign pclk_locked =  pclk_ready;

////////////////////////////////////////////////
// I/O clock for SERDES
//BUFG BUF_pclkx5 (
BUFIO BUFIO_pclkx5 (
  .O(pclkx5), // 1-bit output: Clock output (connect to I/O clock loads).
  .I(mmcm_pclkx5)  // 1-bit input: Clock input (connect to an IBUF or BUFMR).
);
// div clock for SERDES
BUFR #(
   .BUFR_DIVIDE("5"),   // Values: "BYPASS, 1, 2, 3, 4, 5, 6, 7, 8" 
   .SIM_DEVICE("7SERIES")  // Must be set to "7SERIES" 
)
BUFR_pclk (
   .O(pclkx1),     // 1-bit output: Clock output port
   .CE(pclk_ready),   // 1-bit input: Active high, clock enable (Divided modes only)
   .CLR(0), // 1-bit input: Active high, asynchronous clear (Divided modes only)
   .I(mmcm_pclkx5)      // 1-bit input: Clock buffer input driven by an IBUF, MMCM or local intinterconnect
);

////////////////////////////////////////////////
// I/O clock for FAB+DDR
BUFG BUGF_pclk_x5p (
//BUFIO BUGF_pclk_x5p (
  .O(pclkx5p), // 1-bit output: Clock output (connect to I/O clock loads).
  .I(mmcm_pclkx5p)  // 1-bit input: Clock input (connect to an IBUF or BUFMR).
);
BUFG BUGF_pclk_x5n (
//BUFIO BUGF_pclk_x5n (
  .O(pclkx5n), // 1-bit output: Clock output (connect to I/O clock loads).
  .I(mmcm_pclkx5n)  // 1-bit input: Clock input (connect to an IBUF or BUFMR).
);

// for FABLIC
BUFG BUFG_pclk (
  .O(pclk),
  .I(pclkx1)
//  .I(mmcm_pclkx1)
);
// for FABLIC serializer
wire pclkx5_g;
BUFG BUFG_pclkx5 (
  .O(pclkx5_g), // 1-bit output: Clock output (connect to I/O clock loads).
  .I(mmcm_pclkx5)  // 1-bit input: Clock input (connect to an IBUF or BUFMR).
);

//******************************************************************//
// output video timing & pattern generator
//******************************************************************//
wire [7:0] i_vga_r;
wire [7:0] i_vga_g;
wire [7:0] i_vga_b;
wire vga_hblank;
wire vga_vblank;

wire vga_hsync;
wire vga_vsync;
wire vga_disp;

assign vga_disp = ~(vga_hblank|vga_vblank);

// video test patern selector
wire [7:0] test_pattern =
     sw[0]  ? 8'h02 : // sw[0] = on  :   2=GRADARION SCROLL
              8'h01 ; // sw[0] = off :   1=COLOR BAR

videogen
#(
// video timming
 .HPIXELS(HPIXELS), 
 .HFNPRCH(HFNPRCH), 
 .HSYNCPW(HSYNCPW),
 .HBKPRCH(HBKPRCH),
 .HSYNC_POLARITY(HSYNC_POLARITY),
 .VLINES(VLINES),
 .VBKPRCH(VBKPRCH),
 .VSYNCPW(VSYNCPW),
 .VFNPRCH(VFNPRCH),
 .VSYNC_POLARITY(VSYNC_POLARITY)
) videogen_inst (
  .I_RESET(~pclk_ready),
  .TEST_PATTERN(test_pattern), // 0=BLANK , 1=COLOR BAR , 2=GRADARION SCROLL
// VIDEO OUTPUT
  .I_OCLK(pclk),
  .O_R(i_vga_r),    .O_G(i_vga_g),  .O_B(i_vga_b),
  .O_HBLANK(vga_hblank),    .O_VBLANK(vga_vblank),
  .O_HSYNC(vga_hsync),      .O_VSYNC(vga_vsync)
);

//////////////////////////////////////////////////////////////////////
// 1st DVI-D ENCODER MAC
//////////////////////////////////////////////////////////////////////
wire tmds_reset   = ~pclk_locked;

wire [9:0] dvi10_ch0  ,dvi10_ch1  , dvi10_ch2;
wire [4:0] disparity0 ,disparity1 , disparity2; // debug status

dvitx_mac_dbg
//dvi_encoder
dvi_encoder_1
(
    .rstin      (tmds_reset),
// video input
    .clkin      (pclk),      .de         (vga_disp),
    .blue_din   (i_vga_b),   .green_din  (i_vga_g),   .red_din    (i_vga_r),
    .hsync      (vga_hsync), .vsync      (vga_vsync), .preamble    (4'b0000),
// DVI-D output (DVI channel number layout)
    .tmds_data0 (dvi10_ch0),    // DVI-CH0 ( HDMI-CH2 )
    .tmds_data1 (dvi10_ch1),    // DVI-CH1 ( HDMI-CH1 )
    .tmds_data2 (dvi10_ch2),    // DVI-CH2 ( HDMI-CH0 )
    .disparity0(disparity0), .disparity1(disparity1),.disparity2(disparity2)    
);
////////////////////////////////////////////////////////////////
// 1st DVI-D TX-PHY (SERIALIZER,I/O BUFFER)
///////////////////////////////////////////////////////////////
wire [3:0] tmds1_pad_p , tmds1_pad_n;

// TMDS transmitter with debug signal
tmds_tx_phy_xc7 #(
    .SERIALIZER ( "VIVADO_IP"     ),     // SelecIOOSERDESE2VACUI
    .IO_MODE    ( "TMDS_33"       )    
) tmds1_phy (
    // input
    .I_RESET(tmds_reset),
    .I_ICLK(pclk),
    .I_CH0(dvi10_ch0), // blue hsync,vsync
    .I_CH1(dvi10_ch1), // green
    .I_CH2(dvi10_ch2), // read
    .I_PCLK(pclkx1), .I_SCLK(pclkx5),
    .O_TMDS_P(tmds1_pad_p),
    .O_TMDS_N(tmds1_pad_n)
);
// PMOD pin assign
assign {ja[0],ja[1]} = {tmds1_pad_p[2],tmds1_pad_n[2]}; // CH2 (R)
assign {ja[2],ja[3]} = {tmds1_pad_p[0],tmds1_pad_n[0]}; // CH0 (B)
assign {ja[4],ja[5]} = {tmds1_pad_p[1],tmds1_pad_n[1]}; // CH1 (G)
assign {ja[6],ja[7]} = {tmds1_pad_p[3],tmds1_pad_n[3]}; // CHC

//////////////////////////////////////////////////////////////////////
// TMDS1 signal debug status visualizer
//////////////////////////////////////////////////////////////////////
wire [3:0] debug_select1 = 'd0; // fBXpeBl
wire [3:0] debug_select2 = 'd1; // Mω
// wire [3:0] debug_select2 = 'd2; // ̓pXX[

wire [7:0] dvi2_b,dvi2_g,dvi2_r;
tmds_debug tmds1_debug(
	.I_SELECT(debug_select1),
    // low speed senction
    .I_RESET (tmds_reset), .I_CLK   (pclk       ),
    .I_HSYNC ( vga_hsync), .I_VSYNC (vga_vsync  ), .I_DEN(vga_disp ),
    // debug status
    .I_PIXEL8_0(i_vga_b),   .I_PIXEL8_1(i_vga_g),   .I_PIXEL8_2(i_vga_r),
    .I_TMDS10_0(dvi10_ch0), .I_TMDS10_1(dvi10_ch1), .I_TMDS10_2(dvi10_ch2),
    .I_DPTY_0(disparity0),   .I_DPTY_1(disparity1),   .I_DPTY_2(disparity2),
//
	.O_DEN(),                   // passthrough with pipeline delay
	.O_HSYNC() , .O_VSYNC() ,      // passthrough with pipeline delay
	.O_CH0(dvi2_b)   , .O_CH1(dvi2_g) , .O_CH2(dvi2_r)  // selected output
);

wire [7:0] dvi3_b,dvi3_g,dvi3_r;
tmds_debug tmds1_debug_2(
	.I_SELECT(debug_select2),
    // low speed senction
    .I_RESET (tmds_reset), .I_CLK   (pclk       ),
    .I_HSYNC ( vga_hsync), .I_VSYNC (vga_vsync  ), .I_DEN(vga_disp ),
    // debug status
    .I_PIXEL8_0(i_vga_b),   .I_PIXEL8_1(i_vga_g),   .I_PIXEL8_2(i_vga_r),
    .I_TMDS10_0(dvi10_ch0), .I_TMDS10_1(dvi10_ch1), .I_TMDS10_2(dvi10_ch2),
    .I_DPTY_0(disparity0),   .I_DPTY_1(disparity1),   .I_DPTY_2(disparity2),
//
	.O_DEN(),                   // passthrough with pipeline delay
	.O_HSYNC() , .O_VSYNC() ,      // passthrough with pipeline delay
	.O_CH0(dvi3_b)   , .O_CH1(dvi3_g) , .O_CH2(dvi3_r)  // selected output
);

//////////////////////////////////////////////////////////////////////
// 2nd DVI-D
//////////////////////////////////////////////////////////////////////
wire [3:0] tmds2_pad_p,tmds2_pad_n;

// TMDS transmitter
dvitx_xc7 #(
//  .SERIALIZER == "VIVADO_IP"  ),  // SelecIOOSERDESE2VACUI
  .SERIALIZER ( "FAB_DDR"     ),    // HDLLqODDRŃVACUI
  .IO_MODE    ( JB_TMDS_MODE  )     // I/O mode 
) tmdstx_jb (
  // low speed senction
  .I_RESET (tmds_reset), .I_CLK   (pclk       ),
  .I_HSYNC ( vga_hsync), .I_VSYNC (vga_vsync  ), .I_DEN(vga_disp ),
  .I_B     (dvi2_b ), .I_G     (dvi2_g ), .I_R  (dvi2_r ),
  // high speed senction
//    .I_PCLK  (pclkx1     ) , .I_SCLK  (pclkx5     ), // VANbNBUFIOI
  .I_PCLK  (pclk     ) , .I_SCLK  (pclkx5_g     ), // VANbNBUFGI
  .O_TMDS_P(tmds2_pad_p) , .O_TMDS_N(tmds2_pad_n)
);

// PMOD pin assign
pmod_tmds pmod_tmds_jb ( .TMDS_P(tmds2_pad_p) , .TMDS_N(tmds2_pad_n) , .PMOD(jb) );

//////////////////////////////////////////////////////////////////////
// 3rd DVI-D/HDMI TX
//////////////////////////////////////////////////////////////////////
wire [3:0] tmds3_pad_p,tmds3_pad_n;

// TMDS transmitter
dvitx_xc7 #(
  .SERIALIZER ( "FAB_DDR"     ),    // JC TMDS_33ŋ쓮łȂOSERDESE2ł͑IoȂ
  .IO_MODE    ( JC_TMDS_MODE  )     // I/O mode 
) tmdstx_jc (
  // low speed senction
  .I_RESET (tmds_reset), .I_CLK   (pclk     ),
  .I_HSYNC ( vga_hsync), .I_VSYNC (vga_vsync), .I_DEN(vga_disp),
  .I_B     (dvi3_b   ), .I_G     (dvi3_g  ), .I_R  (dvi3_r ),
  // high speed senction
//    .I_PCLK  (pclkx1     ) , .I_SCLK  (pclkx5     ), // VANbNBUFIOI
  .I_PCLK  (pclk     ) , .I_SCLK  (pclkx5_g     ), // VANbNBUFGI
  .O_TMDS_P(tmds3_pad_p) , .O_TMDS_N(tmds3_pad_n)
);

// PMOD pin assign
pmod_tmds pmod_tmds_jc ( .TMDS_P(tmds3_pad_p) , .TMDS_N(tmds3_pad_n) , .PMOD(jc) );

//////////////////////////////////////////////////////////////////////
// timming monitor output
//////////////////////////////////////////////////////////////////////
wire clk1hz;
divider #(
  .WIDTH(26) , .CYCLE(27000000)
) divider1hz (
  .R(1'b0) , .C(clk27m) , .O(clk1hz)
);

///////////////////////////////////////////////////////////////////////
// LED / SW
///////////////////////////////////////////////////////////////////////

assign led[3] = btn[3] | clk1hz;
assign led[2] = btn[2] | sw[2];
assign led[1] = btn[1] | sw[1];
assign led[0] = btn[0] | sw[0];

assign {led0_g,led0_r,led0_b} = 3'b000;
assign {led1_g,led1_r,led1_b} = 3'b000;

endmodule
