//////////////////////////////////////////////////////////////////////////////
//	debug picture generator for TMDS transmitter
//
//
// I_SELEDT
//  0 : disparity
//  1 : num flips
//  2 : input signal
//
//////////////////////////////////////////////////////////////////////////////
`timescale 1 ns / 1ps

module tmds_debug (
	input  wire       I_RESET,            // reset
	input  wire  [3:0] I_SELECT,		   // output selector
	//
	input  wire       I_CLK,              // pixel clock
	input  wire       I_DEN,              // data enable (active area)
	input  wire       I_HSYNC , I_VSYNC , // sync signals
	//
	input  wire [7:0] I_PIXEL8_0 ,I_PIXEL8_1 , I_PIXEL8_2 , // Pixel data Blue,Green,Red
	input  wire [9:0] I_TMDS10_0 ,I_TMDS10_1 , I_TMDS10_2,  // TMDS word data
	input  wire [4:0] I_DPTY_0   ,I_DPTY_1   , I_DPTY_2,    // disparity value
//
	output reg        O_DEN,                   // passthrough with pipeline delay
	output reg        O_HSYNC , O_VSYNC ,      // passthrough with pipeline delay
	output reg [7:0]  O_CH0   , O_CH1 , O_CH2  // selected output
);

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
wire pclk = I_CLK;

//////////////////////////////////////////////////////////////////////
// function
//////////////////////////////////////////////////////////////////////

// number of flips
function [3:0] numFlips10(input [10:0] in);
begin
  numFlips10 = 
    (in[10]^in[9]) + (in[9]^in[8]) + (in[8]^in[7]) + (in[7]^in[6]) + (in[6]^in[5])
   +(in[5]^in[4])  + (in[4]^in[3]) + (in[3]^in[2]) + (in[2]^in[1]) + (in[1]^in[0]);
end
endfunction

// number of flips tu monitor level
function [7:0] flips_to_level(input [3:0] numFlips);
begin
  flips_to_level[7:4] = 4'd10 -numFlips;
  flips_to_level[3:0] = (numFlips=='d10) ? 4'b0000 : 4'b1111 ;
end
endfunction

// absolute (disparity offset level)
function [3:0] abs4(input [4:0] in);
begin
  abs4 = in[4] ? 'd0-in[3:0] : in[3:0];
end
endfunction


//////////////////////////////////////////////////////////////////////
// non-flip data count
//////////////////////////////////////////////////////////////////////
wire [23:0] flip_rgb;

reg tmds_ch0_d10;
reg tmds_ch1_d10;
reg tmds_ch2_d10;

reg [3:0] tmds_ch0_flips;
reg [3:0] tmds_ch1_flips;
reg [3:0] tmds_ch2_flips;

always @(posedge pclk)
begin
    tmds_ch0_d10 <= I_TMDS10_0[0];
    tmds_ch1_d10 <= I_TMDS10_1[1];
    tmds_ch2_d10 <= I_TMDS10_2[2];
    //
    tmds_ch0_flips <= numFlips10( {tmds_ch0_d10 , I_TMDS10_0});
    tmds_ch1_flips <= numFlips10( {tmds_ch1_d10 , I_TMDS10_1});
    tmds_ch2_flips <= numFlips10( {tmds_ch2_d10 , I_TMDS10_2});
end
// R,G,B  converiosn
assign flip_rgb[ 7: 0] = flips_to_level( tmds_ch0_flips );
assign flip_rgb[15: 8] = flips_to_level( tmds_ch1_flips );
assign flip_rgb[23:16] = flips_to_level( tmds_ch2_flips );

//////////////////////////////////////////////////////////////////////
// disparity monitor
//////////////////////////////////////////////////////////////////////
wire [23:0] dpy_rgb;

wire [3:0] dpy_abs0 = abs4(I_DPTY_0);
wire [3:0] dpy_abs1 = abs4(I_DPTY_1);
wire [3:0] dpy_abs2 = abs4(I_DPTY_2);

// disparity warning level
function [7:0] disparity_test(input [3:0] dpy_abs);
begin
        disparity_test =
		(dpy_abs>='d10) ? 8'hff :
		(dpy_abs>='d9)  ? 8'hcf :
		(dpy_abs>='d8)  ? 8'h1f :
		{4'h0,dpy_abs[3:0]};
end
endfunction

// R,G,B  converiosn
assign dpy_rgb[ 7: 0] = disparity_test(I_DPTY_0);
assign dpy_rgb[15: 8] = disparity_test(I_DPTY_1);
assign dpy_rgb[23:16] = disparity_test(I_DPTY_2);

//////////////////////////////////////////////////////////////////////
// ouptut selector
//////////////////////////////////////////////////////////////////////
always @(posedge pclk)
begin
	{O_DEN,O_HSYNC, O_VSYNC} <= {I_DEN,I_HSYNC , I_VSYNC};
	{O_CH2,O_CH1,O_CH0} <= 
		I_SELECT=='d0 ? dpy_rgb  : 
		I_SELECT=='d1 ? flip_rgb : 
		{I_PIXEL8_2,I_PIXEL8_1,I_PIXEL8_0}; // copy
end


endmodule

