//////////////////////////////////////////////////////////////////////////////
//	TMDS encoder
//////////////////////////////////////////////////////////////////////////////
`timescale 1 ns / 1ps

module dvitx_mac_dbg (
input  wire       clkin,          // pixel clock
input  wire       rstin,          // reset
input  wire [7:0] blue_din,       // Blue data in
input  wire [7:0] green_din,      // Green data in
input  wire [7:0] red_din,        // Red data in
input  wire       hsync,          // hsync data
input  wire       vsync,          // vsync data
input  wire       de,             // data enable
//
input  wire [3:0] preamble,       // Adittional data (for HDMI)
//
// TMDS serial otput
output wire [9:0] tmds_data0,
output wire [9:0] tmds_data1,
output wire [9:0] tmds_data2,
// status
output wire [4:0] disparity0,
output wire [4:0] disparity1,
output wire [4:0] disparity2
);		// 10-bit
	
wire 	[9:0]	red ;
wire 	[9:0]	green ;
wire 	[9:0]	blue ;

tmds8b10b encb (
	.rstin	(rstin),.clkin	(clkin),.de(de),
	.din		(blue_din),
	.cin        ({vsync,hsync}),
	.dcnt      	(disparity0),
	.dout		(blue)) ;
tmds8b10b encg (
	.rstin	(rstin),.clkin	(clkin),.de(de),
	.din		(green_din),
	.cin		(preamble[1:0]),
	.dcnt      	(disparity1),
	.dout		(green)) ;
	
tmds8b10b encr (
	.rstin	(rstin),.clkin	(clkin),.de(de),
	.din		(red_din),
	.cin		(preamble[3:2]),
	.dcnt      	(disparity2),
	.dout		(red)) ;

assign tmds_data0 = red;
assign tmds_data1 = green;
assign tmds_data2 = blue;

endmodule

