/****************************************************************************
TMDS encoder MAC+PHY for Xilinx 7series

2021.4 Tatsuyuki Sato

note:
    parameter

    SERIALIZER
        "FAB_SDR"     : generic SDR clock , not suported yet
        "FAB_DDR"     : generic DDR clock
    IO_MODE           : don't care

****************************************************************************/
`timescale 1 ns / 1ps

module dvitx_xc7 #(
    SERIALIZER = "FAB_DDR",    // SERIALIZER  select
    IO_MODE    = "PSEDO_PP"    // I/O PAD MODE
)(
	// MAC section
	input  wire       I_RESET,            // reset
	input  wire       I_CLK,              // pixel clock
	input  wire       I_DEN,              // data enable (active area)
	input  wire       I_HSYNC , I_VSYNC , // sync signals
	input  wire [7:0] I_B , I_G , I_R   , // 8bpp Blue,Green,Red level
	// PHY section
    input           I_PCLK,               // output period clock , x1 I_ICLK
    input           I_SCLK,               // output serial clock , x5 I_SCLK
    output  [3:0]   O_TMDS_P, O_TMDS_N    // output PADs
);

// wire
wire [9:0] tmds_ch0 , tmds_ch1 , tmds_ch2;

// MAC
dvitx_mac mac
(
	.rstin      (I_RESET),  .clkin      (I_CLK    ) , .de         (I_DEN   ),
	.hsync      (I_HSYNC ), .vsync      (I_VSYNC  ) ,
	.blue_din   (I_B     ), .green_din  (I_G      ) , .red_din    (I_R     ),
	.tmds_data0 (tmds_ch0), .tmds_data1 (tmds_ch1 ) , .tmds_data2 (tmds_ch2)
);

// PHY
tmds_tx_phy_xc7 #(
	.SERIALIZER ( SERIALIZER  ),     .IO_MODE    ( IO_MODE     )
) phy (
	// input
	.I_RESET(I_RESET),   	.I_ICLK(I_CLK),
	.I_CH0(tmds_ch0),   	.I_CH1(tmds_ch1),  	.I_CH2(tmds_ch2),
	// output
	.I_PCLK(I_PCLK),  	    .I_SCLK(I_SCLK),
	.O_TMDS_P(O_TMDS_P),    .O_TMDS_N(O_TMDS_N)
);

endmodule
