////////////////////////////////////////////////////////////
// TMDS 8b10b , 2b10b GR[_[
//
// Refer to DVI 1.0 Specification, page 29, Figure 3-5
// 2009NłXilinx APPm[gx[XAdisparitỹoOC
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// module interface
////////////////////////////////////////////////////////////
module tmds8b10b
(
  input            clkin,   // pixel clock input
  input            rstin,   // async. reset input (active high)
  input      [7:0] din,     // 8b data inputs : expect registered
  input      [1:0] cin,     // 2b data inputs : c1,C0
  input            de,      // data(8b) enable input
  output reg [9:0] dout,    // 10b outputs
  output reg [4:0] dcnt     // disparity counter (DC balance)
);
////////////////////////////////////////////////////////////
// functions
////////////////////////////////////////////////////////////

  ///////////////////////////////////////////////////////
  // Stage 1
  // E̓f[^󂯕t
  // E8bit f[^̃rbg '1' ̐𐔂B
  ///////////////////////////////////////////////////////
  reg [3:0] n1_s1; // number of '1' in din
  reg [7:0] din_s1;
  reg       de_s1;
  reg [1:0] c_s1;
  always @ (posedge clkin) begin
    n1_s1 <=#1 din[0] + din[1] + din[2] + din[3] + din[4] + din[5] + din[6] + din[7]; // rbg'1'̐
    {din_s1,de_s1,c_s1} <=#1 {din,de,cin};  // Stage֑
  end

  ///////////////////////////////////////////////////////
  // Stage 2
  // E8 bit -> 9 bit ϊ
  ///////////////////////////////////////////////////////
  wire decision1;
  wire [8:0] q_s2;
  wire       de_s2;
  wire [1:0] c_s2;
  // Zʂ̗אڃrbg]񐔂ȂȂ悤XORXONR牉ZIԁB
  // XNORÍA
  //   1. rbg'1'̐S bit0'0'
  //   2. rbg'1'̐Tȏ
  // ̂ǂ炩
  assign decision1 = (n1_s1 > 4'h4) || ((n1_s1 == 4'h4) & (din_s1[0] == 1'b0));
  // 8bit -> 9bit ϊ
  assign q_s2[0] = din_s1[0];
  assign q_s2[1] = (decision1) ? (q_s2[0] ^~ din_s1[1]) : (q_s2[0] ^ din_s1[1]);
  assign q_s2[2] = (decision1) ? (q_s2[1] ^~ din_s1[2]) : (q_s2[1] ^ din_s1[2]);
  assign q_s2[3] = (decision1) ? (q_s2[2] ^~ din_s1[3]) : (q_s2[2] ^ din_s1[3]);
  assign q_s2[4] = (decision1) ? (q_s2[3] ^~ din_s1[4]) : (q_s2[3] ^ din_s1[4]);
  assign q_s2[5] = (decision1) ? (q_s2[4] ^~ din_s1[5]) : (q_s2[4] ^ din_s1[5]);
  assign q_s2[6] = (decision1) ? (q_s2[5] ^~ din_s1[6]) : (q_s2[5] ^ din_s1[6]);
  assign q_s2[7] = (decision1) ? (q_s2[6] ^~ din_s1[7]) : (q_s2[6] ^ din_s1[7]);
  assign q_s2[8] = (decision1) ? 1'b0 : 1'b1;

  assign {de_s2,c_s2} = {de_s1,c_s1};  // Stage֑

  /////////////////////////////////////////////////////////
  // Stage 3
  // E9 bit f[^̂ bit0-7'0'̐'1'̐̍vZ
  /////////////////////////////////////////////////////////
  reg [3:0] n1sub4_s3;  // '1'̐ - 4 == ('1'̐ -'0'̐)/2 : -4,-3,-2,-1, 0, 1, 2, 3 , 4
  reg       de_s3;
  reg [8:0] q_s3;
  reg [1:0] c_s3;

  always @ (posedge clkin) begin
    // '1'̐ - '0'̐ == '1'̐ - (8-'1'̐) == '1'̐ x2 -8 == ('1'̐ -4)*2
    n1sub4_s3         <= #1 -4'h4 +q_s2[0] +q_s2[1] +q_s2[2] +q_s2[3] +q_s2[4] +q_s2[5] +q_s2[6] +q_s2[7] ;
    {q_s3,de_s3,c_s3} <= #1 {q_s2,de_s2,c_s2}; // Stage֑
  end
  wire [3:0] n0sub4_s3 = 4'h0-n1sub4_s3; // ('0'̐ -'1'̐)/2


  reg [3:0] n1q_m;
  always @ (posedge clkin) begin
    n1q_m  <=#1 q_s2[0] + q_s2[1] + q_s2[2] + q_s2[3] + q_s2[4] + q_s2[5] + q_s2[6] + q_s2[7];
  end
  wire [3:0] n0q_m = 4'h8 - n1q_m;

  /////////////////////////////////////////////////////////
  // Stage 4
  // E9 bit -> 10bitϊ
  /////////////////////////////////////////////////////////
  
  // ߋ̃rbg'0''1'̐̍ĩ݂fBXpeB[lj
  wire cnt_plus  = ~dcnt[4];             // '1'̐ >= '0'̐ (lvXA܂̓[)
  wire cnt_minus =  dcnt[4];             // '1'̐ <  '0'̐ (l}CiX)
  // ̃rbg'0''1'̐̍ifBXpeB[lj
  wire n1q_eq_n0q = (n1sub4_s3==4'h0);  // '1''0'
  wire n1q_ge_n0q = ~n1sub4_s3[3];      // '1'̐ >= '0'̐
  wire n1q_lt_n0q =  n1sub4_s3[3];      // '1'̐ <  '0'̐
  wire [4:0] n1q_sub_n0q = {n1sub4_s3,1'b0}; // '1'̐ - '0'̐i񔽓]Ȋlj
  wire [4:0] n0q_sub_n1q = {n0sub4_s3,1'b0}; // '0'̐ - '1'̐i@]Ȋlj

  // 9b -> 10b ϊ̑IP
  // fBXpeB[[A܂͍̃fBXpeB[[
  assign decision2 = (dcnt == 5'h0) | (n1q_eq_n0q);

  // 9b -> 10b ϊ̑IQ
  // 1.݂̃fBXpeB[vX[ŁA '0''1''1'̕
  // 2.݂̃fBXpeB[}CiXA '1''0'̐
  assign decision3 = (cnt_plus & n1q_ge_n0q) | (cnt_minus & n1q_lt_n0q);

  ///////////////////////////////
  always @ (posedge clkin or posedge rstin) begin
    if(rstin) begin
      dout <= 10'h0;
      dcnt <= 5'h0;
    end else begin
      if (de_s3) begin               // DataEnablel
        // \GAA8bit -> 10bitϊ
        if(decision2) begin        // 1. fBXpeB[[A܂͍̃fBXpeB[[
          dout[9]   <=#1 ~q_s3[8];                           // ]A񔽓]XOR/XNORIrbgŌ
          dout[8]   <=#1  q_s3[8];                           // XOR/XNORIrbg
          dout[7:0] <=#1 (q_s3[8]) ? q_s3[7:0] : ~q_s3[7:0]; // 񔽓]܂͔]
          dcnt <=#1 (~q_s3[8]) ? (dcnt +n0q_sub_n1q) : (dcnt +n1q_sub_n0q); // fBXpeB[lXV

        end else begin
          if(decision3) begin     // 2.]ZDChtgꍇ
            dout[9]   <=#1  1'b1;           // ]Irbg
            dout[8]   <=#1  q_s3[8];        // XOR/XNORIrbg
            dout[7:0] <=#1 ~q_s3[7:0];      // bit0-7 ͔]
            dcnt <=#1 dcnt + {q_s3[8],1'b0} + n0q_sub_n1q; // fBXpeB[lXV
          end else begin         // 3.񔽓]ZDChtgꍇ
            dout[9]   <=#1 1'b0;            // 񔽓]Irbg
            dout[8]   <=#1  q_s3[8];        // XOR/XNORIrbg
            dout[7:0] <=#1  q_s3[7:0];      // bit0-7 ͔񔽓]
            dcnt <=#1 dcnt - {~q_s3[8],1'b0} + n1q_sub_n0q; // fBXpeB[lXV
          end
        end
      end else begin
        // uLOGAA2bit -> 10bitϊ
        case (c_s3)                   // control data(VSYNC,HSYNC)2b10bϊ
          2'b00:   dout <=#1 10'b1101010100;
          2'b01:   dout <=#1 10'b0010101011;
          2'b10:   dout <=#1 10'b0101010100;
          default: dout <=#1 10'b1010101011;
        endcase
        // 2b->10bŔfBXpeB́}1ȉƏ߃fBXpeB͍sȂB
        dcnt <=#1 5'h0;
      end
    end
  end

endmodule
