//-----------------------------------------------------------------------------
// TMDS Transmitter PHY for Xilinx 7 series Standard I/O With DDR buffer 
//------------------------------------------------------------------------------
module tmdstx_xc7_oddr
#(
	parameter IOMODE = "SINGLE_PP"  // "SINGLE_PP" or "SINGLE_OD"
)
(
input reset,
input clkin,
input clkx5p,
input clkx5n, // not use
input [9:0] data_in0,
input [9:0] data_in1,
input [9:0] data_in2,
output [3:0] tmds_p,
output [3:0] tmds_n
);

// ZbgM̈ʑ
reg reset_sync1;
always @(posedge clkin or posedge reset)
begin
	if(reset) reset_sync1 <= 1'b1;
	else reset_sync1 <= 1'b0;
end

// ̓f[^̃z[h
reg [29:0] data_hold;
always @(posedge clkin)
begin
	data_hold <= {data_in2,data_in1,data_in0};
end

// ZbgM̍ē
reg reset_sync2;
always @(posedge clkx5p or posedge reset_sync1)
begin
	if(reset_sync1) reset_sync2 <= 1'b1;
	else reset_sync2 <= 1'b0;
end

// NbNWX^VAU
reg [2:0] sync_timer;
reg [9:0] sreg0,sreg1,sreg2,sreg3;
//wire [9:0] sreg3 = 10'b1111100000;

always @(posedge clkx5p or posedge reset_sync2)
begin
	if(reset_sync2)
	begin 
		sync_timer <= 'd3;
	end else begin
		if(sync_timer>='d4)
		begin
//			sreg3 <= 10'b1111100000;
			sreg3 <= 10'b0000011111;
			sreg2 <= data_hold[29:20];
			sreg1 <= data_hold[19:10];
			sreg0 <= data_hold[ 9: 0];
			sync_timer <= 'd0;
		end else begin
			sreg3[7:0] <= sreg3[9:2];
			sreg2[7:0] <= sreg2[9:2];
			sreg1[7:0] <= sreg1[9:2];
			sreg0[7:0] <= sreg0[9:2];
			sync_timer <= sync_timer+'d1;
		end
	end
end
// ddr output buffers
diff_ddr_out tmds0(.din(sreg0[1:0]),.clk(clkx5p),.outp(tmds_p[0]),.outn(tmds_n[0]));
diff_ddr_out tmds1(.din(sreg1[1:0]),.clk(clkx5p),.outp(tmds_p[1]),.outn(tmds_n[1]));
diff_ddr_out tmds2(.din(sreg2[1:0]),.clk(clkx5p),.outp(tmds_p[2]),.outn(tmds_n[2]));
diff_ddr_out tmdsc(.din(sreg3[1:0]),.clk(clkx5p),.outp(tmds_p[3]),.outn(tmds_n[3]));

endmodule

// ddr output buffer wrapper module
module diff_ddr_out(
	input [1:0] din,
	input clk,
	output outp,   // out_p==out[0] , out_n==out[1]
	output outn
);

ODDR #(.DDR_CLK_EDGE("SAME_EDGE"),.INIT(1'b1)) DDR_p (.Q(outp),.C(clk),.CE(1'b1),.D1( din[0]),.D2( din[1]),.R(1'b0),.S(1'b0));
ODDR #(.DDR_CLK_EDGE("SAME_EDGE"),.INIT(1'b1)) DDR_n (.Q(outn),.C(clk),.CE(1'b1),.D1(~din[0]),.D2(~din[1]),.R(1'b0),.S(1'b0));

endmodule

