////////////////////////////////////////////////////////////
// dispatch改良版
//
// Xilinx APPノートのものにterc4とガードバンド出力を追加したもの
//
// disparityのバグ修正
//
////////////////////////////////////////////////////////////

// PIPELINE BYPASS CONTROL
//`define ASYNC_STAGE0
`define ASYNC_STAGE1 /* force define */
//`define ASYNC_STAGE2

////////////////////////////////////////////////////////////
// TMDS 10b encoder
////////////////////////////////////////////////////////////
module encodeh
#(
	parameter [9:0] vgbpattern = 10'b0100110011,	// video guardband pattern
	parameter [9:0] dgbpattern = 10'b0100110011	// data guardband pattern
)
(
  input            clkin,    // pixel clock input
  input            rstin,    // async. reset input (active high)
  input      [7:0] din,      // data inputs: expect registered
  input      [1:0] cin,      // c1,C0
  input            de,       // de input
// TERC4 data
	input	[3:0]	terc4in,
	input			t4en,
	input			vgben,	// video guardband enable
	input			dgben,	// data guardband enable
//
	output reg [9:0] dout      // data outputs
);
////////////////////////////////////////////////////////////
// functions
////////////////////////////////////////////////////////////

  ///////////////////////////////////////////////////////
  // Stage 0
  //
  // latch input
  // count the number of bit '1'.
  ///////////////////////////////////////////////////////

  ////////////////////////////////////////////////////////////
  // Counting number of 1s and 0s for each incoming pixel
  // component. Pipe line the result.
  // Register Data Input so it matches the pipe lined adder
  // output
  ////////////////////////////////////////////////////////////
`ifdef ASYNC_STAGE0
  wire [3:0] n1d   = din[0] + din[1] + din[2] + din[3] + din[4] + din[5] + din[6] + din[7];
  wire [7:0] din_q = din;
  wire       de_q  = de;
  wire [1:0] c_q   = cin;
  wire [3:0] terc4_q = terc4in;
  wire t4en_q      = t4en; 
  wire vgben_q     = vgben;
  wire dgben_q     = dgben;
`else
  reg [3:0] n1d; // number of '1' in din
  reg [7:0] din_q;
  reg       de_q;
  reg [1:0] c_q;
  reg [3:0] terc4_q;
  reg t4en_q,vgben_q,dgben_q;
  always @ (posedge clkin) begin
    n1d <=#1 din[0] + din[1] + din[2] + din[3] + din[4] + din[5] + din[6] + din[7];
  // pipe line alignment
    din_q <=#1 din;
    de_q  <=#1 de;
    c_q   <=#1 cin;
	terc4_q  <= #1 terc4in;
	t4en_q   <= #1 t4en;
	{vgben_q,dgben_q} <= #1 {vgben,dgben};
	end
`endif

  ///////////////////////////////////////////////////////
  // Stage 1: 8 bit -> 9 bit
  // Refer to DVI 1.0 Specification, page 29, Figure 3-5
  ///////////////////////////////////////////////////////
  wire decision1;
  assign decision1 = (n1d > 4'h4) | ((n1d == 4'h4) & (din_q[0] == 1'b0));
`ifdef ASYNC_STAGE1
  wire [8:0] q_m;
  assign q_m[0] = din_q[0];
  assign q_m[1] = (decision1) ? (q_m[0] ^~ din_q[1]) : (q_m[0] ^ din_q[1]);
  assign q_m[2] = (decision1) ? (q_m[1] ^~ din_q[2]) : (q_m[1] ^ din_q[2]);
  assign q_m[3] = (decision1) ? (q_m[2] ^~ din_q[3]) : (q_m[2] ^ din_q[3]);
  assign q_m[4] = (decision1) ? (q_m[3] ^~ din_q[4]) : (q_m[3] ^ din_q[4]);
  assign q_m[5] = (decision1) ? (q_m[4] ^~ din_q[5]) : (q_m[4] ^ din_q[5]);
  assign q_m[6] = (decision1) ? (q_m[5] ^~ din_q[6]) : (q_m[5] ^ din_q[6]);
  assign q_m[7] = (decision1) ? (q_m[6] ^~ din_q[7]) : (q_m[6] ^ din_q[7]);
  assign q_m[8] = (decision1) ? 1'b0 : 1'b1;
`else
  reg [8:0] q_m;
  always @ (posedge clkin) begin
    q_m[0] <=#1 din_q[0];
    q_m[1] <=#1 (decision1) ? (q_m[0] ^~ din_q[1]) : (q_m[0] ^ din_q[1]);
    q_m[2] <=#1 (decision1) ? (q_m[1] ^~ din_q[2]) : (q_m[1] ^ din_q[2]);
    q_m[3] <=#1 (decision1) ? (q_m[2] ^~ din_q[3]) : (q_m[2] ^ din_q[3]);
    q_m[4] <=#1 (decision1) ? (q_m[3] ^~ din_q[4]) : (q_m[3] ^ din_q[4]);
    q_m[5] <=#1 (decision1) ? (q_m[4] ^~ din_q[5]) : (q_m[4] ^ din_q[5]);
    q_m[6] <=#1 (decision1) ? (q_m[5] ^~ din_q[6]) : (q_m[5] ^ din_q[6]);
    q_m[7] <=#1 (decision1) ? (q_m[6] ^~ din_q[7]) : (q_m[6] ^ din_q[7]);
    q_m[8] <=#1 (decision1) ? 1'b0 : 1'b1;
  end
`endif

  /////////////////////////////////////////////////////////
  // Stage 2: 9 bit calcration
  /////////////////////////////////////////////////////////

`ifdef ASYNC_STAGE2
  wire signed [3:0] n1q_sub_4  = q_m[0] + q_m[1] + q_m[2] + q_m[3] + q_m[4] + q_m[5] + q_m[6] + q_m[7] - 4'h4;
  wire       de_reg     = de_q;
  wire [1:0] c_reg      = c_q;
  wire [8:0] q_m_reg    = q_m;
  wire [3:0] terc4_reg  = terc4_q;
  wire t4en_reg         = t4en_q;
  wire vgben_reg        = vgben_q;
  wire dgben_reg        = dgben_q;
`else
  //reg [3:0] n1q_m, n0q_m; // number of 1s and 0s for q_m
  reg [3:0] n1q_sub_4; // -4,-3,-2,-1, 0, 1, 2, 3 , 4
  reg       de_reg;
  reg [8:0] q_m_reg;
//  reg [1:0] c_reg;
//  reg [3:0] terc4_reg;
//  reg t4en_reg,vgben_reg,dgben_reg;

  always @ (posedge clkin) begin
    //n1q_m  <=#1 q_m[0] + q_m[1] + q_m[2] + q_m[3] + q_m[4] + q_m[5] + q_m[6] + q_m[7];
    //n0q_m  <=#1 4'h8 - (q_m[0] + q_m[1] + q_m[2] + q_m[3] + q_m[4] + q_m[5] + q_m[6] + q_m[7]);
    n1q_sub_4 <= #1 -4'h4 + q_m[0] + q_m[1] + q_m[2] + q_m[3] + q_m[4] + q_m[5] + q_m[6] + q_m[7] ;
// pipe line alignment
    de_reg  <=#1 de_q;
    q_m_reg <=#1 de_q ? q_m : {dgben_q,vgben_q,t4en_q,c_q,terc4_q};
//  q_m_reg <=#1 q_m;
//  terc4_reg  <= #1 terc4_q;
//  c_reg      <=#1 c_q;
//	t4en_reg   <= #1 t4en_q;
//	vgben_reg  <= #1 vgben_q;
//	dgben_reg  <= #1 dgben_q;
  end
  wire [4:0] n0q_sub_4 = 4'h0-n1q_sub_4;
  wire [3:0] terc4_reg = q_m_reg[3:0];
  wire [1:0] c_reg     = q_m_reg[5:4];
  wire t4en_reg        = q_m_reg[6];
  wire vgben_reg       = q_m_reg[7];
  wire dgben_reg       = q_m_reg[8];
`endif

  /////////////////////////////////////////////////////////////////////////
  // Stage 3: 9 bit -> 10 bit
  /////////////////////////////////////////////////////////////////////////
  reg [4:0] cnt; //disparity counter, MSB is the sign bit
  wire decision2, decision3;

  // arithmetic
  wire cnt_plus  = ~cnt[4]; // cnt >= 0
  wire cnt_minus =  cnt[4]; // cnt <  0

  wire n1q_eq_n0q = (n1q_sub_4==4'h0);  // N1{q_m[7:0]} == N0{q_m[7:0]}
  wire n1q_ge_n0q = ~n1q_sub_4[3];      // N1{q_m[7:0]} >= N0{q_m[7:0]}
  wire n1q_lt_n0q =  n1q_sub_4[3];      // N1{q_m[7:0]} <  N0{q_m[7:0]}
  // -4(8:0),-3(7:1),-2(6:2),-1(5:3),0(4:4) -> -8,-6,-4,-2,0
  wire [4:0] n1q_sub_n0q = {n1q_sub_4,1'b0};    // N1{q_m[7:0]} - N0{q_m[7:0]}
  // 0(4:4),1(5:3),2(6:2),3(7:1),4(8:0) -> 0,-2,-4,-6,-8
  wire [4:0] n0q_sub_n1q = {n0q_sub_4,1'b0}; // N0{q_m[7:0]} - N1{q_m[7:0]}
  
//  assign decision2 = (cnt == 5'h0) | (n1q_m == n0q_m);
  assign decision2 = (cnt == 5'h0) | (n1q_eq_n0q);
  /////////////////////////////////////////////////////////////////////////
  // [(cnt > 0) and (N1q_m > N0q_m)] or [(cnt < 0) and (N0q_m > N1q_m)]
//  assign decision3 = (~cnt[4] & (n1q_m > n0q_m)) | (cnt[4] & (n0q_m > n1q_m));
  assign decision3 = (cnt_plus & n1q_ge_n0q) | (cnt_minus & n1q_lt_n0q);
  
  ///////////////////////////////
  // 10-bit out
  // disparity counter
  ///////////////////////////////
  always @ (posedge clkin or posedge rstin) begin
    if(rstin) begin
      dout <= 10'h0;
      cnt <= 5'h0;
    end else begin
      if (de_reg) begin
        // display area
        if(decision2) begin
          dout[9]   <=#1 ~q_m_reg[8]; 
          dout[8]   <=#1 q_m_reg[8]; 
          dout[7:0] <=#1 (q_m_reg[8]) ? q_m_reg[7:0] : ~q_m_reg[7:0];
//          cnt <=#1 (~q_m_reg[8]) ? (cnt + n0q_m - n1q_m) : (cnt + n1q_m - n0q_m);
          cnt <=#1 (~q_m_reg[8]) ? (cnt + n0q_sub_n1q ) : (cnt + n1q_sub_n0q );
        end else begin
          if(decision3) begin
            dout[9]   <=#1 1'b1;
            dout[8]   <=#1 q_m_reg[8];
            dout[7:0] <=#1 ~q_m_reg[7:0];
//            cnt <=#1 cnt + {q_m_reg[8], 1'b0} + (n0q_m - n1q_m);
            cnt <=#1 cnt + {q_m_reg[8], 1'b0} + n0q_sub_n1q;
          end else begin
            dout[9]   <=#1 1'b0;
            dout[8]   <=#1 q_m_reg[8];
            dout[7:0] <=#1 q_m_reg[7:0];
//            cnt <=#1 cnt - {~q_m_reg[8], 1'b0} + (n1q_m - n0q_m);
            cnt <=#1 cnt - {~q_m_reg[8], 1'b0} + n1q_sub_n0q;
          end
        end
      end else begin
        // blank area
 		if(vgben_reg) begin
          dout <=#1 vgbpattern;
		end else if(dgben_reg) begin
          dout <=#1 dgbpattern;
		end else if(t4en_reg) begin
          case (terc4_reg)
            4'b0000: dout <=#1 10'b1010011100;
            4'b0001: dout <=#1 10'b1001100011;
            4'b0010: dout <=#1 10'b1011100100;
            4'b0011: dout <=#1 10'b1011100010;
            4'b0100: dout <=#1 10'b0101110001;
            4'b0101: dout <=#1 10'b0100011110;
            4'b0110: dout <=#1 10'b0110001110;
            4'b0111: dout <=#1 10'b0100111100;
            4'b1000: dout <=#1 10'b1011001100;
            4'b1001: dout <=#1 10'b0100111001;
            4'b1010: dout <=#1 10'b0110011100;
            4'b1011: dout <=#1 10'b1011000110;
            4'b1100: dout <=#1 10'b1010001110;
            4'b1101: dout <=#1 10'b1001110001;
            4'b1110: dout <=#1 10'b0101100011;
            4'b1111: dout <=#1 10'b1011000011;
          endcase
		end else begin
          case (c_reg)
            2'b00:   dout <=#1 10'b1101010100;
            2'b01:   dout <=#1 10'b0010101011;
            2'b10:   dout <=#1 10'b0101010100;
            default: dout <=#1 10'b1010101011;
          endcase
		end
        // clear disparity counter
        cnt <=#1 5'h0;
      end
    end
  end
endmodule
