/****************************************************************************
DVI-D scan converter example for Digilent ZYBO

2021.3-2021.4 Tatsuyuki Sato

note:

o̓eXgp^[LA[h

̓Lv`MI
+-----+-----+-----+---------------------------+
|SW[3]|SW[2]|SW[1]| Lv`\[XI      |
+-----+-----+-----+---------------------------+
| OFF | OFF | OFF | QVGAeXgp^[    |
| OFF | OFF | ON  | SH2A  QVGA 16bpp          |
| OFF | ON  | OFF | RX62N QVGA 16bpp          |
| OFF | ON  | ON  | BLANCA VGA 24bpp          |
+-----+-----+-----+---------------------------+
s蓖Ă 'pmod_capture.v' Q

+------+---------------------------------------+
|SW[0] | o͉摜I                          |
+------+---------------------------------------+
| OFF  | VGAeXgp^[                 |
| ON   | ̓Lv`摜it[obt@j|
+------+---------------------------------------+

+------+---------------------------------------+
|BTN[0]| eXgp^[I                    |
+------+---------------------------------------+
| OFF  | HDJ[o[                          |
| ON   | J[Of[V                  |
+------+---------------------------------------+

+------+---------------------------------------+
|LED[3]| 1Hz NbNgmFpM          |
|LED[2]| TMDS PHY ubN fB[M        |
+------+---------------------------------------+

o̓[hύXƂ܂삵ȂȂ܂B

****************************************************************************/

`timescale 1 ps / 1 ps

/****************************************
 TOP module
****************************************/
module cq_pmod_upscan #
(
//  parameter CAP_WIDTH  = 320,
//  parameter CAP_HEIGHT = 240,
//  parameter CAP_BPP    = 16,         // 6,16
  parameter CAP_WIDTH  = 660,
  parameter CAP_HEIGHT = 480,
  parameter CAP_BPP    = 6,         // 6/12/16
  parameter SERIALIZER = "IPCORE",  // "IPCORE" , "HDODDR"
  parameter JB_MODE = "INPUT"
  
) (
  input  wire clk,      // 125MHz
// HDMI
  output wire hdmi_clk_n,
  output wire hdmi_clk_p,
  output wire [2:0] hdmi_d_p,
  output wire [2:0] hdmi_d_n,
//  output wire hdmi_hpd,
  inout wire hdmi_cec,
  input wire hdmi_hpd,
  output wire hdmi_out_en,
  inout wire hdmi_scl,
  inout wire hdmi_sda,
// VGA output
   output wire [4:0] vga_r,
   output wire [5:0] vga_g,
   output wire [4:0] vga_b,
   output wire vga_hs,
   output wire vga_vs,
// PMOD
  output  wire [3:0] ja_p,ja_n,
//  output wire [3:0] jb_p,
//  output wire [3:0] jb_n,
  input wire [3:0] jb_p,
  input wire [3:0] jb_n,
  input wire [3:0] jc_p,
  input wire [3:0] jc_n,
  input wire [3:0] jd_p,
  input wire [3:0] jd_n,
  input wire [7:0] je,
// buttons
  input  wire [3:0] btn,
// switch
  input  wire [3:0] sw,
// LED
  output  wire [3:0] led
);

/****************************************
 video mode and clock generator defines
 ***************************************/
// choice video mode define
`include "vga_pll_pll.vh"
//`include "uxga_pll_pll.vh"
//`include "sxga_pll_pll.vh"
//`include "fhd_pll_pll.vh"

//******************************************************************//
// 27MHz base clock Generation
//******************************************************************//

// input buffer
wire clk125m;
IBUF clk125m_bufg (.I(clk), .O(clk125m));

// PLL output 
wire pll270m_g;
wire clk27m;
wire clk27m_locked;

// clock generation
wire pll270m;
wire pll27m_locked;
pll125m27m pll125m27m(
  .CLK125M_I(clk125m),
  .CLK27M_O(pll27m),
  .CLK270M_O(pll270m_g),
  .reset(1'b0),
  .locked(pll27m_locked)
);
assign clk27m        = pll27m;
assign clk27m_locked = pll27m_locked;

//******************************************************************//
// pixes clock generation with 7series MMCM
//******************************************************************//
wire mmcm_pclkx5;
wire mmcm_pclkx5p,mmcm_pclkx5n;
wire mmcm_pclkx1;
wire mmcm_clkfb;

// viod output pixel clock
wire pclk_ready;
wire pclk;

// 7series MMCM 
`PCLK_PLL_MODULE  // clk27m25p2m
PCLK_GEN(
  .CLK_IN(clk27m),
  .clkfb_in(mmcm_clkfb),
  .clkfb_out(mmcm_clkfb),
//
  .CLK_X5(mmcm_pclkx5),
  .CLK_X5P(mmcm_pclkx5p),
  .CLK_X5N(mmcm_pclkx5n),
  .CLK_X2(/*mmcm_pclkx2*/),
  .CLK_X1(mmcm_pclkx1),
  .reset(~clk27m_locked || ~pll27m_locked),
  .locked(pclk_ready)
);
assign pclk_locked =  pclk_ready;

//////////////////////////////////////////////
// I/O clock for SERDES
//////////////////////////////////////////////
wire pclkx5_io,pclkx1_io;

// I/O clock for SERDES
BUFIO BUFIO_pclkx5 (
  .O(pclkx5_io), // 1-bit output: Clock output (connect to I/O clock loads).
  .I(mmcm_pclkx5)  // 1-bit input: Clock input (connect to an IBUF or BUFMR).
);
// clocks for SERDES
BUFR #(
   .BUFR_DIVIDE("5"),   // Values: "BYPASS, 1, 2, 3, 4, 5, 6, 7, 8" 
   .SIM_DEVICE("7SERIES")  // Must be set to "7SERIES" 
)
BUFR_pclk (
   .O(pclkx1_io),     // 1-bit output: Clock output port
   .CE(pclk_ready),   // 1-bit input: Active high, clock enable (Divided modes only)
   .CLR(0), // 1-bit input: Active high, asynchronous clear (Divided modes only)
   .I(mmcm_pclkx5)      // 1-bit input: Clock buffer input driven by an IBUF, MMCM or local intinterconnect
);

//////////////////////////////////////////////
// I/O clock for FAB-SERDES
//////////////////////////////////////////////
generate begin
if(SERIALIZER == "FAB_OBUFDS")
begin
// FAB mode ddr-clock
BUFG BUGF_pclk_x5p (
  .O(pclkx5p), // 1-bit output: Clock output (connect to I/O clock loads).
  .I(mmcm_pclkx5p)  // 1-bit input: Clock input (connect to an IBUF or BUFMR).
);
BUFG BUGF_pclk_x5n (
  .O(pclkx5n), // 1-bit output: Clock output (connect to I/O clock loads).
  .I(mmcm_pclkx5n)  // 1-bit input: Clock input (connect to an IBUF or BUFMR).
);
//BUFG BUGF_pclk_x2 (
//  .O(pclkx2), // 1-bit output: Clock output (connect to I/O clock loads).
//  .I(mmcm_pclkx2)  // 1-bit input: Clock input (connect to an IBUF or BUFMR).
//);
end
end endgenerate

//////////////////////////////////////////////
// viode pixel clock for Fablic
//////////////////////////////////////////////
BUFG pclk_g (.I(pclkx1_io), .O(pclk));

//******************************************************************//
// qvga internal reference clock
//******************************************************************//
wire clk6m25;
wire clk6m25_rclk;
BUFR #(
   .BUFR_DIVIDE("4"),   // Values: "BYPASS, 1, 2, 3, 4, 5, 6, 7, 8" 
   .SIM_DEVICE("7SERIES")  // Must be set to "7SERIES" 
)
BUFR_clk6m25 (
   .O(clk6m25_rclk),     // 1-bit output: Clock output port
   .CE(pclk_ready),   // 1-bit input: Active high, clock enable (Divided modes only)
   .CLR(0), // 1-bit input: Active high, asynchronous clear (Divided modes only)
   .I(mmcm_pclkx1)      // 1-bit input: Clock buffer input driven by an IBUF, MMCM or local interconnect
);
assign clk6m25 = clk6m25_rclk;

//******************************************************************//
// QVGA test pattern for upscan test
//******************************************************************//
wire        test_vblank;
wire        test_hblank;
wire [ 7:0] test_g , test_r , test_b;
wire        test_clk;
wire        test_vs;
wire        test_hs;
wire        test_de;

// video test patern selector
wire [7:0] qvga_pattern =
     btn[0] ? 8'h02 : // on  :   2=GRADARION SCROLL
              8'h01 ; // off :   1=COLOR BAR

videogen
#(
// video timming
 .HPIXELS(HPIXELS/2),   .HFNPRCH(HFNPRCH/2),   .HSYNCPW(HSYNCPW/2),  .HBKPRCH(HBKPRCH/2),  .HSYNC_POLARITY(1'b1), 
 .VLINES(VLINES/2),     .VBKPRCH(VBKPRCH/2),   .VSYNCPW(VSYNCPW/2),  .VFNPRCH(VFNPRCH/2),  .VSYNC_POLARITY(1'b1)
) qvga_tmggen_inst (
  .I_RESET(~pclk_ready),
// TEST PATTERN select
  .TEST_PATTERN(qvga_pattern), // 0=BLANK , 1=COLOR BAR , 2=GRADARION SCROLL
// VIDEO OUTPUT
  .I_OCLK(test_clk),
  .O_R(test_r),  .O_G(test_g), .O_B(test_b),
  .O_HBLANK(test_hblank), .O_VBLANK(test_vblank),
  .O_HSYNC(test_hs),   .O_VSYNC(test_vs)
);
assign test_de  = ~(test_hblank|test_vblank);
assign test_clk = clk6m25;

//******************************************************************//
// input video select switch
//******************************************************************//
wire [2:0] input_sel = sw[3:1];

//******************************************************************//
// input video signal selector
//******************************************************************//
// clock frequency for capture clock regeneration
localparam PCLK_DIV = 270000000/10;

// clock regeneration
wire pclk_recovery;
wire [25:0] pclk_mul;
wire [25:0] pclk_phase;

// DE regeneration
wire dataEnabeRegen;
wire [11:0] hstart , vstart , vscale;
wire [11:0] hsize  , vsize  , hscale;

// capture input signal
wire [7:0] cap_g , cap_r , cap_b;
wire cap_hsync  , cap_vsync;
wire cap_disp;
wire cap_clk;
wire cap_clk_g;  // grobal clock

wire [7:0] jb_in;

pmod_capture #(
    .PCLK_DIV(PCLK_DIV)
) input_selector (
  .I_SEL(input_sel) ,
// internal input
  .I_G(test_g),.I_R(test_r),.I_B(test_b),
  .I_CLK(test_clk),
  .I_DE(test_de),
  .I_VSYNC(test_vs),
  .I_HSYNC(test_hs),
// external input
  .I_J0(jb_in),
  .I_J1({jc_n[3],jc_p[3],jc_n[2],jc_p[2],jc_n[1],jc_p[1],jc_n[0],jc_p[0]}),
  .I_J2({jd_n[3],jd_p[3],jd_n[2],jd_p[2],jd_n[1],jd_p[1],jd_n[0],jd_p[0]}),
  .I_J3(je[7:0]),
//
  .O_PCLK_REGEN(pclk_recovery),
  .O_PCLKDIV(),
  .O_PCLKMUL(pclk_mul),     .O_PCLKPHASE(pclk_phase),       // regenerate clock
//
  .O_DE_REGEN(dataEnabeRegen),                              // windows regenerate  
  .O_HSTART(hstart),        .O_HSIZE(hsize),                // window H-parameter
  .O_VSTART(vstart),        .O_VSIZE(vsize),                // window V-parameter
  .O_HSCALE(hscale),        .O_VSCALE(vscale),              // scale select
// resampling
//  .I_RESAMPLE_CLOCK(),
//  .O_CLK(),
//  .O_DE(),
// output
  .O_G(cap_g),.O_R(cap_r),.O_B(cap_b),
  .O_CLK(cap_clk),       .O_DE(cap_disp),
  .O_VSYNC(cap_vsync),   .O_HSYNC(cap_hsync)
);

//******************************************************************//
// pixel clock re-generation
//******************************************************************//
BUFG bufg_capclk (.I(cap_clk), .O(cap_clk_g));
wire cap_regen_disp = cap_disp;

//******************************************************************//
// video frame buffer
//******************************************************************//

// write bus
wire vram_wclk = cap_clk_g;
wire vram_wen;
wire [22: 0] vram_waddr;
wire [23: 0] vram_wdata24;

//read bus
wire vram_rclk = pclk;
wire vram_ren;
wire [22: 0] vram_raddr;
wire [23: 0] vram_rdata24;

frame_buffer #(
	.DATA_SIZE(CAP_WIDTH*CAP_HEIGHT),
	.ADDR_DEPTH(23),
	.DATA_DEPTH(CAP_BPP)
) frame_buffer_inst (
	.I_RESET(~pclk_ready),
	// WRITE BUS
	.I_WCLK(vram_wclk),  .I_WEN(vram_wen), .I_WADDR(vram_waddr), .I_WDATA(vram_wdata24),
	// READ BUS
	.I_RCLK(vram_rclk),  .I_REN(vram_ren), .I_RADDR(vram_raddr), .O_RDATA(vram_rdata24)
);

//******************************************************************//
// input video capture
//******************************************************************//
wire [11:0] vram_width;
wire [11:0] vram_height;
videocapture
#(
	.DATA_SIZE(CAP_WIDTH*CAP_HEIGHT),
	.ADDR_DEPTH(23), // address depth
	.DATA_DEPTH(24)	 // data depth
) videocapture_inst (
	.I_RESET(~pclk_ready),
	// PIXEL INPUT DATA
	.I_CLK(cap_clk_g),
	.I_DISP(cap_regen_disp),
	.I_VSYNC(cap_vsync),
	.I_PIXEL({cap_g,cap_r,cap_b}),
	// INFO
	.O_WIDTH(vram_width),
	.O_HEIGHT(vram_height),
	// VIDEO RAM
	.O_WEN(vram_wen),
	.O_ADDR(vram_waddr),
	.O_DATA(vram_wdata24)
);

//******************************************************************//
// output video timing & test pattern generator
//******************************************************************//
wire [7:0] i_vga_r , i_vga_g , i_vga_b;
wire vga_hsync   , vga_vsync;
wire vga_hblank  , vga_vblank;
wire vga_disp;

assign vga_disp = ~(vga_hblank|vga_vblank);

// video test patern selector
wire [7:0] test_pattern =
     btn[0] ? 8'h02 : //  on  :   2=GRADARION SCROLL
              8'h01 ; //  off :   1=COLOR BAR

videogen
#(
// video timming
 .HPIXELS(HPIXELS), 
 .HFNPRCH(HFNPRCH), 
 .HSYNCPW(HSYNCPW),
 .HBKPRCH(HBKPRCH),
 .HSYNC_POLARITY(HSYNC_POLARITY),
 .VLINES(VLINES),
 .VBKPRCH(VBKPRCH),
 .VSYNCPW(VSYNCPW),
 .VFNPRCH(VFNPRCH),
 .VSYNC_POLARITY(VSYNC_POLARITY)
) videogen_inst (
  .I_RESET(~pclk_ready),
// TEST PATTERN select
  .TEST_PATTERN(test_pattern), // 0=BLANK , 1=COLOR BAR , 2=GRADARION SCROLL
// VIDEO OUTPUT
  .I_OCLK(vram_rclk),
  .O_R(i_vga_r),
  .O_G(i_vga_g),
  .O_B(i_vga_b),
  .O_HBLANK(vga_hblank),
  .O_VBLANK(vga_vblank),
  .O_HSYNC(vga_hsync),
  .O_VSYNC(vga_vsync)
);

//******************************************************************//
// scaler & vram scan
//******************************************************************//
wire dscan_disp;
wire dscan_hsync;
wire dscan_vsync;
wire [23:0] dscan_grb;

videoscaler_ii #(
	.DATA_SIZE(CAP_WIDTH*CAP_HEIGHT),
	.ADDR_DEPTH(23), // address depth
	.DATA_DEPTH(24)	 // data depth
) scaler (
	.I_RESET(~pclk_ready),
	// format selector
	// VIDEO timming input
	.I_CLK(vram_rclk),    .I_DISP(vga_disp),
	.I_VSYNC(vga_vsync),  .I_HSYNC(vga_hsync),
	// SCALE
	// input format selector
	.I_WIDTH(vram_width),   .I_HEIGHT(vram_height),
    .I_HPOS( 'd0  ),        .I_VPOS( 'd0 ),           
	.I_HSCALE(hscale),       .I_VSCALE(vscale),
//	.I_HSCALE(2*256),       .I_VSCALE(2*256),
	// VIDEO RAM input
	.O_REN(vram_ren),
	.O_ADDR(vram_raddr),
	.I_DATA(vram_rdata24), 
	// VIDEO RAM otuput
	.O_PIXEL(dscan_grb),
	.O_HSYNC(dscan_hsync), .O_VSYNC(dscan_vsync),
	.O_DISP(dscan_disp)
);

///////////////////////////////////////////////////////////////////////////
// output video selector
///////////////////////////////////////////////////////////////////////////
wire fbuf_enable = sw[0];

wire [3+23:0] out_rgb = 
  fbuf_enable ? {dscan_disp, dscan_hsync , dscan_vsync , dscan_grb[23:0]} :        // upscan
                {vga_disp  , vga_hsync   , vga_vsync   , i_vga_g,i_vga_r,i_vga_b}; // test pattern

wire          out_vsync = out_rgb[24];
wire          out_hsync = out_rgb[25];
wire          out_disp  = out_rgb[26];

//////////////////////////////////////////////////////////////////////
// DVI-D ENCODER MAC
//////////////////////////////////////////////////////////////////////
wire [9:0] dvi10_ch0,dvi10_ch1,dvi10_ch2;

dvitx_mac dvi_mac_instr
(
    .rstin      (~pclk_locked),
// video input
    .clkin      (pclk),              .de         ( out_disp),
    .hsync      (~out_hsync),        .vsync      (~out_vsync),
    .blue_din   (out_rgb[ 7: 0]),    .green_din  (out_rgb[23:16]),   .red_din    (out_rgb[15: 8]),
// DVI-D output (DVI channel number layout)
    .tmds_data0 (dvi10_ch0),   .tmds_data1 (dvi10_ch1),   .tmds_data2 (dvi10_ch2)
);

////////////////////////////////////////////////////////////////
// TMDS ENCODER PHY
///////////////////////////////////////////////////////////////
wire phy_reset = ~pclk_locked;
wire phy_ready = ~phy_reset;

wire [3:0] tmds1_pad_p;
wire [3:0] tmds1_pad_n;

generate begin
if(SERIALIZER == "IPCORE")
begin
// remap data to IP core dat array
wire    [29:0] data_out_from_device =
{//    CH2          CH1          CH0
    dvi10_ch2[9],dvi10_ch1[9],dvi10_ch0[9], // last out
    dvi10_ch2[8],dvi10_ch1[8],dvi10_ch0[8], // 9th out
    dvi10_ch2[7],dvi10_ch1[7],dvi10_ch0[7],
    dvi10_ch2[6],dvi10_ch1[6],dvi10_ch0[6],
    dvi10_ch2[5],dvi10_ch1[5],dvi10_ch0[5],
    dvi10_ch2[4],dvi10_ch1[4],dvi10_ch0[4],
    dvi10_ch2[3],dvi10_ch1[3],dvi10_ch0[3],
    dvi10_ch2[2],dvi10_ch1[2],dvi10_ch0[2],
    dvi10_ch2[1],dvi10_ch1[1],dvi10_ch0[1], // 2nd out
    dvi10_ch2[0],dvi10_ch1[0],dvi10_ch0[0]  // 1st out
};
TMDS_TX tmds_driver // Vivado IP Catalog
(
// slow clock
    .clk_reset          (~pclk_locked)      ,   .clk_div_in(pclkx1_io),
    .data_out_from_device(data_out_from_device),
// fast clock
    .io_reset           (phy_reset)         ,   .clk_in             (pclkx5_io),
    .clk_to_pins_p      (tmds1_pad_p[3])    ,   .clk_to_pins_n      (tmds1_pad_n[3]),
    .data_out_to_pins_p (tmds1_pad_p[2:0])  ,   .data_out_to_pins_n (tmds1_pad_n[2:0])
);
end else begin

wire [3:0] tmds_ddr_p,tmds_ddr_n; // DDR regster output
tmdstx_xc7_oddr tmds_serialzier(
.reset(phy_reset),
.clkin(pclk),
.clkx5p(pclkx5p),.clkx5n(pclkx5n),
.data_in0(dvi10_ch0),
.data_in1(dvi10_ch1),
.data_in2(dvi10_ch2),
.tmds_p(tmds_ddr_p),.tmds_n(tmds_ddr_n)
);
// TMDS_33 differencial output
OBUFDS TMDS0 (.I(tmds_ddr_p[0]), .O(tmds1_pad_p[0]), .OB(tmds1_pad_n[0])) ;
OBUFDS TMDS1 (.I(tmds_ddr_p[1]), .O(tmds1_pad_p[1]), .OB(tmds1_pad_n[1])) ;
OBUFDS TMDS2 (.I(tmds_ddr_p[2]), .O(tmds1_pad_p[2]), .OB(tmds1_pad_n[2])) ;
OBUFDS TMDSC (.I(tmds_ddr_p[3]), .O(tmds1_pad_p[3]), .OB(tmds1_pad_n[3])) ;
end
end endgenerate

////////////////////////////////////////////////////////////////
// HDMI OUTPUT ASSIGN
///////////////////////////////////////////////////////////////
generate begin
  if(JB_MODE=="HDMI")
  begin
    // JB adapter out
    assign {jb_p[3],jb_p[0],jb_p[2],jb_p[1]}  = tmds1_pad_p[3:0];
    assign {jb_n[3],jb_n[0],jb_n[2],jb_n[1]}  = tmds1_pad_n[3:0];
    // onBoard HDMI dummy output
    OBUFDS TMDS0_DMY (.I(1'b0), .O(hdmi_d_p[0]), .OB(hdmi_d_n[0])) ;
    OBUFDS TMDS1_DMY (.I(1'b0), .O(hdmi_d_p[1]), .OB(hdmi_d_n[1])) ;
    OBUFDS TMDS2_DMY (.I(1'b0), .O(hdmi_d_p[2]), .OB(hdmi_d_n[2])) ;
    OBUFDS TMDS3_DMY (.I(1'b0), .O(hdmi_clk_p), .OB(hdmi_clk_n)) ;
  end else begin
    // onBoard HDMI output 
    assign {hdmi_clk_p,hdmi_d_p[2:0]}  = tmds1_pad_p[3:0];
    assign {hdmi_clk_n,hdmi_d_n[2:0]}  = tmds1_pad_n[3:0];
  end
  // JB dummy TMDS select
  if(JB_MODE=="TMDS_DUMMY")
  begin
    // JB dummy output
    OBUFDS TMDS0_DMY (.I(1'b0), .O(jb_p[0]), .OB(jb_n[0])) ;
    OBUFDS TMDS1_DMY (.I(1'b0), .O(jb_p[1]), .OB(jb_n[1])) ;
    OBUFDS TMDS2_DMY (.I(1'b0), .O(jb_p[2]), .OB(jb_n[2])) ;
    OBUFDS TMDS3_DMY (.I(1'b0), .O(jb_p[3]), .OB(jb_n[3])) ;
  end
  // JB input select
  if(JB_MODE=="INPUT")
  begin
    assign jb_in = {jb_n[3],jb_p[3],jb_n[2],jb_p[2],jb_n[1],jb_p[1],jb_n[0],jb_p[0]};
  end else begin
    assign jb_in = 8'h00;
  end
end endgenerate

///////////////////////////////////////////////////////////////////////
// onboard HDMI : other signals
///////////////////////////////////////////////////////////////////////
// output enable
assign hdmi_out_en = 1'b1;
// DDC I2C , CEC-control
assign {hdmi_scl,hdmi_sda,hdmi_cec} = 3'bzzz;

//////////////////////////////////////////////////////////////////////
// clock timming monitor output
//////////////////////////////////////////////////////////////////////
wire clk1hz;
divider #(
  .WIDTH(26) , .CYCLE(27000000)
) divider1hz (
  .R(1'b0) , .C(clk27m) , .O(clk1hz)
);

///////////////////////////////////////////////////////////////////////
// Analog VGA monitor output
///////////////////////////////////////////////////////////////////////

// analog rgb clock
wire argb_clk;
BUFG bufg_argb_clk (.I(pclk), .O(argb_clk));

reg [7:0] analog_g,analog_r,analog_b;
reg analog_hsync,analog_vsync;

always @(posedge argb_clk)
begin
  {analog_g,analog_r,analog_b} <= vga_disp ? out_rgb : 24'h000000;
  analog_hsync <= ~out_hsync;
  analog_vsync <= ~out_vsync;
end
assign vga_g  = analog_g[7:2];
assign vga_r  = analog_r[7:3];
assign vga_b  = analog_b[7:3];
assign vga_vs = analog_vsync;
assign vga_hs = analog_hsync;

///////////////////////////////////////////////////////////////////////
// unused pmod
///////////////////////////////////////////////////////////////////////
assign ja_p = {4'b0000};
assign ja_n = {4'b0000};

///////////////////////////////////////////////////////////////////////
// LED / SW
///////////////////////////////////////////////////////////////////////
assign led[3] = clk1hz;
assign led[2] = phy_ready;
assign led[1] = 1'b0;
assign led[0] = btn[3:0]!='d0;

endmodule
