//******************************************************************
// Generic DualPort RAM
//******************************************************************

`timescale 1 ps / 1 ps

module dpram
#(
	parameter DATA_SIZE     = 320*200,	// maximum pixels
	parameter ADDR_DEPTH	= 16,		// address depth
	parameter DATA_DEPTH	= 16		// data depth
)(
	input  wire I_RESET,
	// WRITE BUS
	input  wire	I_WCLK,
	input  wire	I_WEN,
	input  wire	[ADDR_DEPTH-1: 0] I_WADDR,
	input  wire	[DATA_DEPTH-1: 0] I_WDATA,
	// READ BUS
	input  wire	I_RCLK,
	input  wire	I_REN,
	input  wire	[ADDR_DEPTH-1: 0] I_RADDR,
	output wire	[DATA_DEPTH-1: 0] O_RDATA
);

///////////////////////////////////////////////////////////////////////////
// DualPort RAM
///////////////////////////////////////////////////////////////////////////

//(* clock_buffer = "IBUFG" *)
reg  [DATA_DEPTH-1: 0]	dpram [DATA_SIZE-1:0];

wire [DATA_DEPTH-1: 0]	clear_data = {DATA_DEPTH-1{1'b0}};
wire [DATA_DEPTH-1: 0]	write_data   = I_RESET ? clear_data : I_WDATA;
wire					write_enable = I_RESET ? 1'b1       : I_WEN && (I_WADDR < DATA_SIZE);

// write port
always @ (posedge I_WCLK)
begin
	if(write_enable)
	begin
		dpram[I_WADDR]  <= write_data;
	end
end

// read port
reg  [DATA_DEPTH-1: 0]	read_data;
always @ (posedge I_RCLK)
begin
	if(I_REN)
	begin
		read_data <= dpram[I_RADDR];
	end
end
assign O_RDATA = read_data;

endmodule
