//******************************************************************
// VIDEO FRAME BUFFER , internal resized RAM
//******************************************************************

`timescale 1 ps / 1 ps

module frame_buffer
#(
	parameter DATA_SIZE    = 320*200,	// vram size
	parameter ADDR_DEPTH	= 16,		// vram address depth
	parameter DATA_DEPTH	= 16		// vram data depth : 24,16,12,6
)(
	input  wire I_RESET,
	// WRITE BUS
	input  wire	I_WCLK,
	input  wire	I_WEN,
	input  wire	[ADDR_DEPTH-1: 0] I_WADDR,
	input  wire	[23: 0] I_WDATA,
	// READ BUS
	input  wire	I_RCLK,
	input  wire	I_REN,
	input  wire	[ADDR_DEPTH-1: 0] I_RADDR,
	output wire [23: 0] O_RDATA
);

///////////////////////////////////////////////////////////////////////////
// data width conversion
///////////////////////////////////////////////////////////////////////////
// remap vram data
wire [DATA_DEPTH-1: 0] vram_wdata;
wire [DATA_DEPTH-1: 0] vram_rdata;
generate
begin
  if(DATA_DEPTH==6) // RGB 2:2:2
  begin
    assign vram_wdata[5:4] = I_WDATA[23:22];
    assign vram_wdata[3:2] = I_WDATA[15:14];
    assign vram_wdata[1:0] = I_WDATA[ 7: 6];
    assign O_RDATA[23:16] = {4{vram_rdata[ 5:4]}};
    assign O_RDATA[15: 8] = {4{vram_rdata[ 3:2]}};
    assign O_RDATA[ 7: 0] = {4{vram_rdata[ 1:0]}};
  end
  if(DATA_DEPTH==12) // RGB 4:4:4
  begin
    assign vram_wdata[11: 8] = I_WDATA[23:20];
    assign vram_wdata[ 7: 4] = I_WDATA[15:12];
    assign vram_wdata[ 3: 0] = I_WDATA[ 7: 4];
    assign O_RDATA[23:16] = {2{vram_rdata[11:8]}};
    assign O_RDATA[15: 8] = {2{vram_rdata[ 7:4]}};
    assign O_RDATA[ 7: 0] = {2{vram_rdata[ 3:0]}};
  end
  if(DATA_DEPTH==16) // RGB 5:6:5
  begin
    assign vram_wdata[15:10] = I_WDATA[23:18];
    assign vram_wdata[ 9: 5] = I_WDATA[15:11];
    assign vram_wdata[ 4: 0] = I_WDATA[7:3  ];
    assign O_RDATA[23:16] = {vram_rdata[15:10],vram_rdata[15:14]};
    assign O_RDATA[15: 8] = {vram_rdata[ 9: 5],vram_rdata[ 9: 7]};
    assign O_RDATA[ 7: 0] = {vram_rdata[ 4: 0],vram_rdata[ 4: 2]};
  end
  if(DATA_DEPTH==24) // RGB 8:8:8
  begin
    assign vram_wdata = I_WDATA;
    assign O_RDATA    = vram_rdata;
  end
end
endgenerate

///////////////////////////////////////////////////////////////////////////
// Video RAM
///////////////////////////////////////////////////////////////////////////
dpram
#(
	.DATA_SIZE(DATA_SIZE), 		// maximum pixels
	.ADDR_DEPTH(ADDR_DEPTH),	// address depth
	.DATA_DEPTH(DATA_DEPTH) 	// data depth
) videoram (
	.I_RESET(I_RESET),
	// WRITE BUS
	.I_WCLK(I_WCLK),    .I_WEN(I_WEN), 	.I_WADDR(I_WADDR), 	.I_WDATA(vram_wdata),
	// READ BUS
	.I_RCLK(I_RCLK),   	.I_REN(I_REN), 	.I_RADDR(I_RADDR), 	.O_RDATA(vram_rdata)
);

endmodule
