//******************************************************************
// video scaler
// AA{
//******************************************************************

`timescale 1 ps / 1 ps

module videoscaler_ii
#(
	parameter DATA_SIZE    = 320*200,	// maximum pixel data
	parameter ADDR_DEPTH	= 16,		// address depth
	parameter DATA_DEPTH	= 16,		// data depth
	parameter [23:0] BORDER_COLOR = 24'h808080 // border color code
)(
	input  wire	I_RESET,
	// format selector
	input  wire	[11: 0] I_WIDTH,
	input  wire	[11: 0] I_HEIGHT,
	input  wire	[11: 0] I_HPOS,
	input  wire	[11: 0] I_VPOS,
	// VIDEO timming input
	input  wire	I_CLK,
	input  wire	I_DISP,
	input  wire	I_VSYNC,
	input  wire	I_HSYNC,
	// SCALING
	input  wire	[11:0] I_HSCALE,		// H scale 1*256 or 2*256
	input  wire	[11:0] I_VSCALE,		// V scale 1*256 or 2*256
	// VIDEO RAM input
	output wire	O_REN,
	output wire	[ADDR_DEPTH-1: 0] O_ADDR,
	input  wire	[DATA_DEPTH-1: 0] I_DATA,
	// VIDEO PIXEL otuput
	output wire	[DATA_DEPTH-1: 0] O_PIXEL,
	output wire	O_HSYNC,
	output wire	O_VSYNC,
	output wire	O_DISP
);

///////////////////////////////////////////////////////////////////////////
// V base address
///////////////////////////////////////////////////////////////////////////

// vvram acccess
reg [ADDR_DEPTH:0] raddr;

// stage 1 vram-request
reg rcycle;
reg vsync_in;
reg hsync_in;
// stage 2
reg disp_cycle;
reg vsync_out;
reg hsync_out;

// horizonal start address
reg [ADDR_DEPTH-1:0] vaddr;

// scale counter
reg [1:0] vstretch_count;
reg [1:0] hstretch_count;

// vsync detent signal
wire vsync_start = vsync_in && ~I_VSYNC;
wire hsync_start = hsync_in && ~I_HSYNC;

always @ (posedge I_CLK)
begin
	if(I_RESET)
	begin
		vsync_in       <= 1'b0;
		hsync_in       <= 1'b0;
		vsync_out      <= 1'b0;
		hsync_out      <= 1'b0;
		raddr          <= 'd0;
		rcycle         <= 'd0;
		disp_cycle     <= 'd0;
		vaddr          <= 'd0;
		vstretch_count <= 'd0;
		hstretch_count <= 'd0;
	end else begin
		// capture input signals
		vsync_in <= I_VSYNC;
		hsync_in <= I_HSYNC;
		rcycle   <= I_DISP;
		
		// vram address calcrator
		if(vsync_start)
		begin
			// 
			vaddr          <= 'd0;
			raddr          <= 'd0;
			vstretch_count <= I_VSCALE[11:8] -'d1; // XP[
		end else begin
			// XL
			if(hsync_start)
			begin
				// 
				hstretch_count <= I_HSCALE[11:8]-'d1;   //  XP[
				if(vstretch_count>0)
				begin
					// X^JԂ
					vstretch_count <= vstretch_count -'d1; // XP[JEg
					raddr          <= vaddr;               // AhXX^擪ɖ߂
				end else begin
					// ɂPXebvړ
					vstretch_count <= I_VSCALE[11:8] -'d1;
					vaddr          <= raddr; // update H-POS
					if(vaddr[11:0]!=raddr[11:0])
					begin
						//ړ
						vaddr <= vaddr + I_WIDTH; // next raster
						raddr <= vaddr + I_WIDTH; // next raster
					end
    			end
			end else begin
				// XL
				if(rcycle)
				begin
					// \
					if(hstretch_count>0)
					begin
						//{p
						hstretch_count <= hstretch_count-'d1;
					end else begin
						//ړ
						hstretch_count <= I_HSCALE[11:8]-'d1;
						raddr <= raddr + 'd1;
					end
				end
			end
		end
		// pCvC
		disp_cycle <= rcycle;
		vsync_out  <= vsync_in;
		hsync_out  <= hsync_in;
	end
end

///////////////////////////////////////////////////////////////////////////
// output assign
///////////////////////////////////////////////////////////////////////////
// VRAM access request
assign O_REN  = I_DISP/*rcycle*/ && (raddr < DATA_SIZE);
assign O_ADDR = raddr;

// VRAM accept & through out
assign O_PIXEL = I_DATA;
assign O_DISP  = disp_cycle;
assign O_HSYNC = hsync_out;
assign O_VSYNC = vsync_out;

endmodule
